package com.boman.gen.service;

import java.util.Collections;
import java.util.List;

import com.boman.common.core.constant.UserConstants;
import com.boman.common.core.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.boman.common.core.text.Convert;
import com.boman.gen.domain.GenTableColumn;
import com.boman.gen.mapper.GenTableColumnMapper;

/**
 * 业务字段 服务层实现
 *
 * @author ruoyi
 */
@Service
public class GenTableColumnServiceImpl implements IGenTableColumnService {
    @Autowired
    private GenTableColumnMapper genTableColumnMapper;

    /**
     * 查询业务字段列表
     *
     * @param tableId 业务字段编号
     * @return 业务字段集合
     */
    @Override
    public List<GenTableColumn> selectGenTableColumnListByTableId(Long tableId) {
        return genTableColumnMapper.selectGenTableColumnListByTableId(tableId);
    }

    /**
     * 查询业务字段列表
     *
     * @param tableIdList 业务字段编号List
     * @return 业务字段集合
     */
    @Override
    public List<GenTableColumn> listByTableIdList(List<Long> tableIdList) {
        if (null == tableIdList || tableIdList.isEmpty()) {
            return Collections.emptyList();
        }

        return genTableColumnMapper.listByTableIdList(tableIdList);
    }

    /**
     * 新增业务字段
     *
     * @param genTableColumn 业务字段信息
     * @return 结果
     */
    @Override
    public int insertGenTableColumn(GenTableColumn genTableColumn) {
        return genTableColumnMapper.insertGenTableColumn(genTableColumn);
    }

    /**
     * 修改业务字段
     *
     * @param genTableColumn 业务字段信息
     * @return 结果
     */
    @Override
    public int updateGenTableColumn(GenTableColumn genTableColumn) {
        return genTableColumnMapper.updateGenTableColumn(genTableColumn);
    }

    /**
     * 删除业务字段对象
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    @Override
    public int deleteGenTableColumnByIds(String ids) {
        return genTableColumnMapper.deleteGenTableColumnByIds(Convert.toLongArray(ids));
    }

    /**
     * 校验column_name是否重复
     *
     * @param genTableColumn
     * @return
     */
    @Override
    public String checkColumnNameUnique(GenTableColumn genTableColumn) {
        Long columnId = StringUtils.isNull(genTableColumn.getColumnId()) ? -1L : genTableColumn.getColumnId();
        GenTableColumn column = genTableColumnMapper.checkColumnNameUnique(genTableColumn.getColumnName());
        if (StringUtils.isNotNull(column) && column.getColumnId().longValue() != columnId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }
}