package com.boman.system.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.boman.common.core.utils.SecurityUtils;
import com.boman.system.common.FormDataConstant;
import com.boman.system.common.MainTableRecord;
import com.boman.system.common.RowResult;
import com.boman.system.mapper.StandardlyMapper;
import com.boman.system.service.IBaseSaveService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.sql.Timestamp;
import java.util.List;
import java.util.Locale;

/**
 * @author shiqian
 * @description
 * @date 2021年03月22日 11:25
 **/
@Component
public class BaseSaveServiceImpl implements IBaseSaveService {

    private static final Logger LOGGER = LoggerFactory.getLogger(BaseSaveServiceImpl.class);

    @Autowired
    private StandardlyMapper mapper;

    /**
     * 功能描述: 保存一行
     *
     * @param tableName         tableName
     * @param pkName            主键名称
     * @param maxId             最大的SEQ
     * @param row               要保存的数据
     * @param allColumnNameList 这是此表的所有字段属性名称，用来判断是否所有UPDATE_BY和UPDATE_TIME
     * @return com.boman.system.common.RowResult
     */
    @Override
    public RowResult insertRow(String tableName, String pkName, Long maxId, MainTableRecord row, List<String> allColumnNameList) {
        JSONObject commitData = row.getMainData().getCommitData();
        Timestamp currentTime = new Timestamp(System.currentTimeMillis());
        if (allColumnNameList.contains(FormDataConstant.UPDATE_BY)
                || allColumnNameList.contains(FormDataConstant.UPDATE_BY.toLowerCase())) {
            commitData.put(FormDataConstant.UPDATE_BY, SecurityUtils.getUserId());
        }
        if (allColumnNameList.contains(FormDataConstant.UPDATE_TIME)
                || allColumnNameList.contains(FormDataConstant.UPDATE_TIME.toLowerCase())) {
            commitData.put(FormDataConstant.UPDATE_TIME, SecurityUtils.getUserId());
        }

        // 新增
        if (row.getId() > 0) {
            commitData.put("CREATE_TIME", currentTime);
            commitData.put("CREATE_BY", SecurityUtils.getUserId());
        }

        commitData.put(pkName.toUpperCase(), maxId);
        int ret = mapper.insert(tableName, commitData);
        if (ret > 0) {
            commitData.put("successCnt", ret);
            return RowResult.ok("保存成功", commitData);
        }

        return RowResult.error("失败");
    }

}
